/****************************************************************************
*  This file is part of the WLAN-Ethernut driver.
*
*  Copyright (c) 2004 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************
*  History:
*
*  07.02.04  mifi   First Version for the XC9572-PC84 ONLY!
*                   The BASE_ADDRESS is 0x8800!!
*  xx.xx.04  mifi   Works now with the state machine
****************************************************************************/
#define __PCMCIA_C__

/*
 * The PCMCIA interface works with a Xilinx XC9572-PC84 CPLD.
 * All the lines like CE1, CE2, WE, OE and so on must be control
 * separate by the latch.
 *
*/

#include <iom128v.h>
#include <macros.h>

#include <sys/atom.h>

#include "compiler.h"

#include "typedefs.h"

#include "pcmcia.h"

/*==========================================================*/
/*  DEFINE: All Structures and Common Constants             */
/*==========================================================*/
typedef enum {
  TYPE_IO = 0,
  TYPE_MEM
} MEMORY_TYPE;

/*==========================================================*/
/*  DEFINE: Definition of all local Data                    */
/*==========================================================*/

/*==========================================================*/
/*  DEFINE: Definition of all local Procedures              */
/*==========================================================*/
/************************************************************/
/*  WriteXXX                                                */
/************************************************************/
static void WriteXXX(WORD wAddress, WORD wData, MEMORY_TYPE eIO)
{
  BYTE bLow;
  BYTE bHigh;
  BYTE bStatus;
  BYTE bTimeout = 255;
    
  NutEnterCritical();
  
  bLow  = LOBYTE(wData);
  bHigh = HIBYTE(wData);
  
  /*
   * Write the data to the CPLD
   */
  WriteHiByte = bHigh;
    
  if (eIO == TYPE_MEM) {
    wAddress += MEM_BASE_ADDRESS;
  } else {
    wAddress += IO_BASE_ADDRESS;
  }  
  *((volatile BYTE*)wAddress) = bLow;
    
  /*
   * Wait for the CPLD
   */
  bStatus = CtrlReg;
  while( ((bStatus & CTRL_READY) == 0) && (bTimeout != 0) ){
    NOP();
    bStatus = CtrlReg;    
    bTimeout--;
  }
  
  NutExitCritical();
}

/************************************************************/
/*  ReadXXX                                                 */
/************************************************************/
static WORD ReadXXX(WORD wAddress, MEMORY_TYPE eIO)
{
  BYTE bStatus;
  BYTE bTimeout = 255;  
  WORD wData;

  NutEnterCritical();
  
  /*
   * Start the read cycle
   */  
  if (eIO == TYPE_MEM) {
    wAddress += MEM_BASE_ADDRESS;
  } else {
    wAddress += IO_BASE_ADDRESS;
  }
  
  bStatus = *((volatile BYTE*)wAddress);
    
  /*
   * Wait for the CPLD
   */
  bStatus = CtrlReg;
  while( ((bStatus & CTRL_READY) == 0) && (bTimeout != 0) ){
    NOP();
    bStatus = CtrlReg;    
    bTimeout--;
  }
  
  wData  = (BYTE)ReadHiByte;
  wData  = wData << 8;
  wData |= ReadLoByte;

  NutExitCritical();

  return (wData);
}

/*==========================================================*/
/*  DEFINE: All code exported                               */
/*==========================================================*/
/************************************************************/
/*  pcmcia_WriteMem                                         */
/************************************************************/
void pcmcia_WriteMem(WORD wAddress, WORD wData)
{
  WriteXXX(wAddress, wData, TYPE_MEM);
}

/************************************************************/
/*  pcmcia_WriteReg                                         */
/************************************************************/
void pcmcia_WriteReg(WORD wAddress, WORD wData)
{
  BYTE bLow;
  BYTE bHigh;
  BYTE bStatus;
  BYTE bTimeout = 255;
      
  cli();
  
  bLow  = LOBYTE(wData);
  bHigh = HIBYTE(wData);
  
  /*
   * Write the data to the CPLD
   */
  NOP(); 
  WriteHiByte = bHigh;
    
  wAddress += IO_BASE_ADDRESS;
  *((volatile BYTE*)wAddress) = bLow;
    
  /*
   * Wait for the CPLD
   */
  NOP(); 
  bStatus = CtrlReg;
  while( ((bStatus & CTRL_READY) == 0) && (bTimeout != 0) ){
    NOP();
    bStatus = CtrlReg;    
    bTimeout--;
  }
  
  sei();
}

/************************************************************/
/*  pcmcia_ReadMem                                          */
/************************************************************/
WORD pcmcia_ReadMem(WORD wAddress)
{
  return (ReadXXX(wAddress, TYPE_MEM));
}

/************************************************************/
/*  pcmcia_ReadReg                                          */
/************************************************************/
WORD pcmcia_ReadReg(WORD wAddress)
{
  return (ReadXXX(wAddress, TYPE_IO));
}



void pcmcia_WriteData0(WORD wData)
{
  BYTE bLow;
  BYTE bHigh;
  BYTE bStatus;
  BYTE bTimeout;
  WORD wAddress;
        
  bTimeout = 255;             
  wAddress = 0x36;  
    
  cli();
  
  bLow  = LOBYTE(wData);
  bHigh = HIBYTE(wData);
  
  /*
   * Write the data to the CPLD
   */
  NOP(); 
  WriteHiByte = bHigh;
    
  wAddress += IO_BASE_ADDRESS;
  *((volatile BYTE*)wAddress) = bLow;
    
  /*
   * Wait for the CPLD
   */
  NOP(); 
  bStatus = CtrlReg;
  while( ((bStatus & CTRL_READY) == 0) && (bTimeout != 0) ){
    NOP();
    bStatus = CtrlReg;    
    bTimeout--;
  }
    
  sei();
}
